import bluej.extensions2.*;
import bluej.extensions2.editor.JavaEditor;
import bluej.extensions2.editor.TextLocation;
import javafx.event.*;
import javafx.scene.control.*;
import javafx.stage.StageStyle;

class MenuBuilder extends MenuGenerator
{
    private BPackage curPackage;
    private BClass curClass;
    private BObject curObject;
    private EventHandler menPopupAction = menuPopupAction();
    private EventHandler menEditClassAction = menuEditClassAction();

    public MenuItem getToolsMenuItem(BPackage aPackage)
    {
        MenuItem mi = new MenuItem();
        mi.setText("Click Tools");
        mi.setId("Tools menu:");
        mi.setOnAction(menPopupAction);
        return mi;
    }

    public MenuItem getPackageMenuItem(BPackage bPackage) {
        MenuItem mi = new MenuItem();
        mi.setText("Click Package");
        mi.setId("Package menu:");
        mi.setOnAction(menPopupAction);
        return mi;
    }

    public MenuItem getClassMenuItem(BClass aClass)
    {
        MenuItem mi1 = new MenuItem();
        mi1.setText("Click Class");
        mi1.setId("Class menu:");
        mi1.setOnAction(menPopupAction);

        MenuItem mi2 = new MenuItem();
        mi2.setText("Add comment");
        mi2.setOnAction(menEditClassAction);

        Menu m = new Menu("Simple Extension");
        m.getItems().addAll(mi1, mi2);
        return m;
    }

    public MenuItem getObjectMenuItem(BObject anObject)
    {
        MenuItem mi = new MenuItem();
        mi.setText("Click Object");
        mi.setId("Object menu:");
        mi.setOnAction(menPopupAction);

        return mi;
    }

    // These methods will be called when
    // each of the different menus are about to be invoked.
    public void notifyPostToolsMenu(BPackage bp, MenuItem mi)
    {
        System.out.println("Post on Tools menu");
        curPackage = bp;
        curClass = null;
        curObject = null;
    }

    public void notifyPostClassMenu(BClass bc, MenuItem mi)
    {
        System.out.println("Post on Class menu");
        curPackage = null;
        curClass = bc;
        curObject = null;
    }

    public void notifyPostObjectMenu(BObject bo, MenuItem mi)
    {
        System.out.println("Post on Object menu");
        curPackage = null;
        curClass = null;
        curObject = bo;
    }

    public void notifyPostPackageMenu(BPackage bp, MenuItem menuItem) {
        System.out.println("Post on Package menu");
        curPackage = bp;
        curClass = null;
        curObject = null;
    }

    // A utility method which pops up a dialog detailing the objects
    public EventHandler menuPopupAction()
    {
        return actionEvent ->
        {
            try
            {
                if (curObject != null)
                    curClass = curObject.getBClass();
                if (curClass != null)
                    curPackage = curClass.getPackage();

                String msg = ((MenuItem) actionEvent.getSource()).getId();
                if (curPackage != null)
                    msg += "\nCurrent Package = " + curPackage;
                if (curClass != null)
                    msg += "\nCurrent Class = " + curClass;
                if (curObject != null)
                    msg += "\nCurrent Object = " + curObject;

                Alert dlg = new Alert(Alert.AlertType.NONE, msg, ButtonType.OK);
                dlg.initStyle(StageStyle.UTILITY);
                dlg.showAndWait();
            } catch (Exception exc)
            {
                exc.printStackTrace();
            }
        };
    }

    // A utility method which writes text to a class
    public EventHandler menuEditClassAction()
    {
        return actionEvent ->
        {
            try
            {
                if (curObject != null)
                    curClass = curObject.getBClass();

                if(curClass != null)
                    addComment(curClass);
            } catch (Exception exc)
            {
                exc.printStackTrace();
            }
        };
    }

    // A method to add a comment at the end of the current class, using the Editor API
    private void addComment(BClass curClass)
    {
        JavaEditor classEditor = null;
        try
        {
            classEditor = curClass.getJavaEditor();
        }
        catch (Exception e) { }
        if(classEditor == null)
        {
            System.out.println("Can't create Java Editor for " + curClass);
            return;
        }

        int textLen = classEditor.getTextLength();
        TextLocation lastLine = classEditor.getTextLocationFromOffset(textLen);
        lastLine.setColumn(0);
        // The TextLocation now points before the first character of the last
        // line of the current text
        // which we'll assume contains the closing } bracket for the class
        classEditor.setText(lastLine, lastLine, "// Comment added by SimpleExtension\n");
    }
}